python lisa.py -dd ~/data/medical/orig/sliver07/training/liver-orig001.mhd -d --roi [[37,180],[36,410],[25,403]]
python lisa.py -dd ~/data/medical/orig/sliver07/training/liver-orig002.mhd -d --roi [[0,64],[49,471],[11,444]]
python lisa.py -dd ~/data/medical/orig/sliver07/training/liver-orig003.mhd -d --roi [[4,78],[36,373],[27,366]]
python lisa.py -dd ~/data/medical/orig/sliver07/training/liver-orig004.mhd -d --roi [[37,208],[78,443],[31,385]]
python lisa.py -dd ~/data/medical/orig/sliver07/training/liver-orig005.mhd -d --roi [[85,302],[48,470],[0,305]]
